from enum import Enum
from typing import Optional


class CampaignStatus(Enum):
    NOT_STARTED = "Not Started"
    SCHEDULED = "Scheduled"
    RUNNING = "Running"
    COMPLETED = "Completed"
    TERMINATED = "Terminated"

    @classmethod
    def from_str(cls, value: str) -> Optional["CampaignStatus"]:
        for member in cls:
            if member.value.lower() == value.lower():
                return member
        return None


class ContactStatus(Enum):
    NOT_STARTED = "not started"
    RINGING = "ringing"
    DIALING = "dialing"
    ANSWERED = "answered"
    HUNGUP = "hungup"
    UNANSWERED = "unanswered"
    FAILED = "failed"
    TERMINATED = "terminated"
    COMPLETED = "completed"

    @classmethod
    def from_str(cls, value: str) -> Optional["ContactStatus"]:
        for member in cls:
            if member.value.lower() == value.lower():
                return member
        return None


class Services(Enum):
    SMS = "SMS"
    PHONE = "PHONE"
    SMS_PHONE = "SMS & PHONE"
    SURVEY = "SURVEY"

    @classmethod
    def from_str(cls, value: str) -> Optional["Services"]:
        for member in cls:
            if member.value.lower() == value.lower():
                return member
        return None


class Category(Enum):
    TEXT = "Text"
    AUDIO = "Audio"

    @classmethod
    def from_str(cls, value: str) -> Optional["Category"]:
        for member in cls:
            if member.value.lower() == value.lower():
                return member
        return None


class FailoverTarget(Enum):
    NOT_STARTED = "not started"
    HUNGUP = "hungup"
    UNANSWERED = "unanswered"
    FAILED = "failed"
    COMPLETED = "completed"

    @classmethod
    def from_str(cls, value: str) -> Optional["FailoverTarget"]:
        for member in cls:
            if member.value.lower() == value.lower():
                return member
        return None


class SurveyContentType(Enum):
    SAY = "say"
    PLAY = "play"
    INFO = "info"

    @classmethod
    def from_str(cls, value: str) -> Optional["FailoverTarget"]:
        for member in cls:
            if member.value.lower() == value.lower():
                return member
        return None


class SurveyResponseType(Enum):
    SPEECH = "raw_record"
    DTMF = "dtmf"
    NONE = "none"

    @classmethod
    def from_str(cls, value: str) -> Optional["FailoverTarget"]:
        for member in cls:
            if member.value.lower() == value.lower():
                return member
        return None


from datetime import datetime
from typing import TYPE_CHECKING, List, Optional, Union

from .contact import Contact
from .dialer import Dialer
from .enums import CampaignStatus, Category, FailoverTarget, Services
from .survey_step import SurveyStep
from .voice import Voice

if TYPE_CHECKING:
    from .client import TingTingClient

class Campaign:
    @classmethod
    def from_data(cls, client:"TingTingClient",data:dict) -> "Campaign":
        if isinstance(data.get("voice"),dict):
            voice = Voice.from_data(client,data.get("voice"))
            voiceId = voice.id
        else:
            voice = None
            voiceId:int = data.get("voice")
            
        failoverTarget:List[Optional[FailoverTarget]] = []
        for status in data.get("failover_target"):
            failoverTarget.append(FailoverTarget.from_str(status))
        
        schedule = data.get("schedule")
        if schedule:
            schedule = datetime.fromisoformat(schedule)
        
        return Campaign(
            client = client,
            id = data.get("id"),
            name = data.get("name"),
            services = Services.from_str(data.get("services")),
            status = CampaignStatus.from_str(data.get("status")),
            smsMessage = data.get("sms_message"),
            message = data.get("message"),
            description = data.get("description"),
            schedule = schedule,
            audioFile = data.get("audio_file"),
            bulkFile = data.get("bulk_file"),
            category = Category.from_str(data.get("category")),
            dialers = data.get("user_phone"),
            contactCount = data.get("campaign_action_count"),
            progressPercent = data.get("progress_percent"),
            updatedAt = data.get("updated_at"),
            creditLimit = data.get("credit_limit"),
            voice = voice,
            voiceId = voiceId,
            draft = data.get("draft"),
            failoverTarget = failoverTarget,
            lengthFactor = data.get("lengthFactor"),
        )

    def __init__(self,
        client:"TingTingClient",
        id: Optional[int] = None,
        name: str = "Test Campaign",
        services: Services = Services.PHONE,
        status: CampaignStatus = CampaignStatus.NOT_STARTED,
        smsMessage: str = "default sms message",
        message: str = "default message",
        description: str = "default description",
        schedule: Optional[datetime] = None,
        audioFile: Optional[str] = None,
        bulkFile: Optional[str] = None,
        category: Category = Category.TEXT,
        dialers: Union[List[int],List[Dialer]] = None,
        contactCount: int = 0,
        progressPercent: int = 0,
        updatedAt: datetime = None,
        creditLimit: int = 0,
        voiceId: Optional[int] = None,
        voice: Optional[Voice] = None,
        draft: bool = False,
        failoverTarget:List[Optional[FailoverTarget]] = None,
        lengthFactor:float = 1,
        surveySteps: Optional[List[SurveyStep]] = None
    ):
        self._client = client
        self.id = id
        self.name = name
        self.services = services
        self.status = status
        self.smsMessage = smsMessage
        self.message = message
        self.description = description
        self.schedule = schedule
        self.audioFile = audioFile
        self.bulkFile = bulkFile
        self.category = category
        self.dialers = dialers or []
        self.contactCount = contactCount
        self.progressPercent = progressPercent
        self.updatedAt = updatedAt
        self.creditLimit = creditLimit
        self.voiceId = voice.id if voice else voiceId
        self._voice = voice
        self.draft = draft
        self.failoverTarget = failoverTarget or []
        self.lengthFactor = lengthFactor
        self.surveySteps = surveySteps or []

    def __str__(self)->str:
        return f"{self.name}"
    
    def json(self)->dict:
        return {
            "id":self.id,
            "name":self.name,
            "services":self.services.value,
            "status":self.status.value,
            "sms_message":self.smsMessage,
            "message":self.message,
            "description":self.description,
            "schedule":self.schedule.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z" if self.schedule else None,
            "audio_file":self.audioFile,
            "bulk_file":self.bulkFile,
            "category":self.category.value,
            "user_phone":[(dialer.id if type(dialer) is Dialer else dialer) for dialer in self.dialers],
            "campaign_action_count":self.contactCount,
            "progress_percent":self.progressPercent,
            "updated_at":self.updatedAt,
            "credit_limit":self.creditLimit,
            "voice":self.voiceId,
            "draft":self.draft,
            "failover_target":[target.value for target in self.failoverTarget if target],
            "length_factor":self.lengthFactor,
            "steps":[step.json() for step in self.surveySteps],
        }
    
    def update_from(self, other: "Campaign"):
        self.id = other.id
        self.name = other.name
        self.services = other.services
        self.status = other.status
        self.smsMessage = other.smsMessage
        self.message = other.message
        self.description = other.description
        self.schedule = other.schedule
        self.audioFile = other.audioFile
        self.bulkFile = other.bulkFile
        self.category = other.category
        self.dialers = other.dialers
        self.contactCount = other.contactCount
        self.progressPercent = other.progressPercent
        self.updatedAt = other.updatedAt
        self.creditLimit = other.creditLimit
        self.voiceId = other.voiceId
        self._voice = other._voice
        self.draft = other.draft
        self.failoverTarget = other.failoverTarget
        self.lengthFactor = other.lengthFactor


    def delete(self):
        pass

    def save(self):
        if self.id:
            return self.update()
        return self.create()
    
    def get_voice(self) -> Voice:
        if not self._voice:
            self._voice = self._client.get_voice_by_id(self.voiceId)
        return self._voice

    def _get_files(self) -> dict:
        files = {}
        for idx, step in enumerate(self.surveySteps):
            if step.audioFile:
                files[str(idx)] = step.audioFile
        return files

    def create(self) -> "Campaign":
        json = self.json()
        voice = self.get_voice()
        if voice:
            json["voice"] = voice.json()
        else:
            del json["voice"]
        
        if self.services == Services.SURVEY:
            files = self._get_files()
            created = self._client.create_survey_campaign(json,files)
        else:
            created = self._client.create_campaign(json)
        self.update_from(created)
        return self

    def update(self):
        pass


    def test_voice(self)->str:
        return self._client.test_voice_campaign(self)

    def run(self):
        if self.services == Services.SURVEY:
            return self._client.run_survey(self)
        return self._client.run_campaign(self)

    def stop(self):
        pass

    def get_report(self)->str:
        pass

    def add_contact(self,contact: Contact) -> Contact:
        contact.campaignId = self.id
        return contact.create()

    def add_contact_from_file(self,file):
        self._client.add_contact_file(self,file)

    def add_survey_step(self,surveyStep:SurveyStep):
        surveyStep.order = self.surveySteps.__len__()
        self.surveySteps.append(surveyStep)



import os
import sys
import unittest

from dotenv import load_dotenv

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from tingting.client import TingTingClient, User

load_dotenv()

BASE_URL = os.environ.get("BASE_URL")
TOKEN = os.environ.get("TOKEN")

class TestTingTingClient(unittest.TestCase):
    def test_get_user(self):
        client = TingTingClient(BASE_URL,TOKEN)
        user = client.get_user()

        assert type(user) is User
        assert type(user.id) is int



import uuid
from typing import IO, TYPE_CHECKING, Optional

from .enums import SurveyContentType, SurveyResponseType

if TYPE_CHECKING:
    from .client import TingTingClient
    
class SurveyStep:
    @classmethod
    def from_data(cls, client:"TingTingClient",data:dict) -> "SurveyStep":
        return SurveyStep(
            client = client,
            id = data.get("id"),
            survey = data.get("survey"),
            order = data.get("order"),
            contentType = data.get("content_type"),
            textOrUrl = data.get("text_or_url"),
            responseType = data.get("response_type"),
            inputName = data.get("input_name"),
            audioFile = data.get("audio_file"),
        )

    def __init__(self,
        client:"TingTingClient",
        id: Optional[int] = None,
        survey: Optional[int] = None,
        order: int = 0,
        contentType: SurveyContentType = SurveyContentType.SAY,
        textOrUrl: str = "",
        responseType: SurveyResponseType = SurveyResponseType.DTMF,
        inputName: str = "field_1",
        audioFile: Optional[IO[bytes]] = None,
    ):
        self._client = client
        self.id = id
        self.survey = survey
        self.order = order
        self.contentType = contentType
        self.textOrUrl = textOrUrl
        self.responseType = responseType
        self.inputName = inputName
        self.audioFile = audioFile
    
    def __str__(self) -> str:
        return f"{self.inputName}"
    
    def json(self) -> dict:
        return {
            "id": self.id or str(uuid.uuid4()),
            "survey": self.survey,
            "order": self.order,
            "content_type": self.contentType.value,
            "text_or_url": self.textOrUrl,
            "response_type": self.responseType.value,
            "input_name": self.inputName,
            "audio_file": None,
        }


from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from .client import TingTingClient


class User:
    @classmethod
    def from_data(cls, client:"TingTingClient",data:dict) -> "User":
        return User(
            client = client,
            id = data.get("id"),
            firstName = data.get("user",{}).get("first_name"),
            lastName = data.get("user",{}).get("last_name"),
            username = data.get("user",{}).get("username"),
            email = data.get("user",{}).get("email"),
            profilePicture = data.get("profile_picture"),
            contactNo = data.get("contact_no"),
            address = data.get("address"),
            isVerifiedContact = data.get("is_verified_contact"),
        )

    def __init__(self,
        client:"TingTingClient",
        id:Optional[int] = None,
        firstName: str = "",
        lastName: str = "",
        username: str = "",
        email: str = "",
        profilePicture: str = "",
        contactNo: str = "",
        address: str = "",
        isVerifiedContact: bool = False,
    ):
        self._client = client
        self.id = id
        self.firstName = firstName
        self.lastName = lastName
        self.username = username
        self.email = email
        self.profilePicture = profilePicture
        self.contactNo = contactNo
        self.address = address
        self.isVerifiedContact = isVerifiedContact
    
    def __str__(self):
        return f"{self.username}"
    
    def json(self):
        return {
            "id": self.id,
            "first_name": self.firstName,
            "last_name": self.lastName,
            "username": self.username,
            "email": self.email,
            "profile_picture": self.profilePicture,
            "contact_no": self.contactNo,
            "address": self.address,
            "is_verified_contact": self.isVerifiedContact,
        }
        



import os
import sys
import unittest
from datetime import datetime, timedelta, timezone

from dotenv import load_dotenv

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from tingting import (
    Campaign,
    CampaignStatus,
    Category,
    FailoverTarget,
    Services,
    TingTingClient,
)

load_dotenv()

BASE_URL = os.environ.get("BASE_URL")
TOKEN = os.environ.get("TOKEN")

class TestTingTingClient(unittest.TestCase):
    def test_create_campaign(self):
        client = TingTingClient(BASE_URL,TOKEN)
        campaign = Campaign(
            client=client,
            name="Test Campaign Create",
        ).create()

        assert type(campaign) is Campaign
        assert type(campaign.id) is int

    def test_create_phone_campaign(self):
        client = TingTingClient(BASE_URL,TOKEN)
        dialers = client.get_dialers()
        voices = client.get_voices()
        assert voices.__len__() > 0
        voice = voices[0]

        campaign = Campaign(
            client=client,
            name = "Test Create Phone Campaign",
            services = Services.PHONE,
            status = CampaignStatus.NOT_STARTED,
            message = "Phone Campaign Message",
            category = Category.TEXT,
            dialers = dialers,
            voice = voice,
            draft = False,
            lengthFactor = 1,
        ).create()

        assert type(campaign) is Campaign
        assert type(campaign.id) is int

    def test_create_sms_campaign(self):
        client = TingTingClient(BASE_URL,TOKEN)
        dialers = client.get_dialers()

        campaign = Campaign(
            client = client,
            name = "Test Create SMS Campaign",
            services = Services.SMS,
            status = CampaignStatus.NOT_STARTED,
            smsMessage = "SMS Campaign Message",
            category = Category.TEXT,
            dialers = dialers,
            draft = False,
        ).create()

        assert type(campaign) is Campaign
        assert type(campaign.id) is int

    def test_create_phone_and_sms_campaign(self):
        client = TingTingClient(BASE_URL,TOKEN)
        dialers = client.get_dialers()
        voices = client.get_voices()
        assert voices.__len__() > 0
        voice = voices[0]

        campaign = Campaign(
            client = client,
            name = "Test Create Phone and SMS Campaign",
            services = Services.SMS_PHONE,
            status = CampaignStatus.NOT_STARTED,
            message = "Phone and SMS campaign Phone Message",
            smsMessage = "Phone and SMS campaign SMS Message",
            category = Category.TEXT,
            dialers = dialers,
            voice = voice,
            draft = False,
            lengthFactor = 1,
            failoverTarget = [
                FailoverTarget.HUNGUP,
                FailoverTarget.UNANSWERED,
                FailoverTarget.FAILED,
            ],
        ).create()

        assert type(campaign) is Campaign
        assert type(campaign.id) is int

    def test_create_scheduled_campaign(self):
        client = TingTingClient(BASE_URL,TOKEN)
        dialers = client.get_dialers()
        voices = client.get_voices()
        assert voices.__len__() > 0
        voice = voices[0]
        schedule = datetime.now(timezone.utc) + timedelta(minutes=5)

        campaign = Campaign(
            client=client,
            name = "Test Create Scheduled Campaign",
            services = Services.PHONE,
            status = CampaignStatus.NOT_STARTED,
            message = "Scheduled Campaign Message",
            category = Category.TEXT,
            dialers = dialers,
            voice = voice,
            draft = False,
            lengthFactor = 1,
            schedule = schedule,
        ).create()

        assert type(campaign) is Campaign
        assert type(campaign.id) is int
        assert abs((campaign.schedule - schedule).total_seconds()) < 10




import os
import sys
import unittest

from dotenv import load_dotenv

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from tingting import (
    Campaign,
    Contact,
    Services,
    SurveyContentType,
    SurveyResponseType,
    SurveyStep,
    TingTingClient,
)

load_dotenv()

BASE_URL = os.environ.get("BASE_URL")
TOKEN = os.environ.get("TOKEN")

class TestTingTingClient(unittest.TestCase):
    def test_survey_launch(self):
        client = TingTingClient(BASE_URL,TOKEN)
        dialers = client.get_dialers()
        voices = client.get_voices()
        assert voices.__len__() > 0
        voice = voices[0]

        file1 = open("sample_audio.mp3","rb")

        campaign = Campaign(
            client = client,
            name = "Test Survey Launch",
            services = Services.SURVEY,
            dialers = dialers,
            voice = voice,
            draft = False,
            lengthFactor = 1,
            surveySteps=[
                SurveyStep(
                    order=0,
                    client = client,
                    contentType = SurveyContentType.SAY,
                    responseType = SurveyResponseType.DTMF,
                    textOrUrl = "Hello Step1",
                    inputName = "field_1",
                ),
                SurveyStep(
                    order=1,
                    client = client,
                    contentType = SurveyContentType.SAY,
                    responseType = SurveyResponseType.SPEECH,
                    textOrUrl = "Hello Step2",
                    inputName = "field_2",
                ),
                SurveyStep(
                    order=2,
                    client = client,
                    contentType = SurveyContentType.PLAY,
                    responseType = SurveyResponseType.DTMF,
                    inputName = "field_3",
                    audioFile = file1,
                ),
            ]
        ).create()
        file1.close()
        assert type(campaign) is Campaign
        assert type(campaign.id) is int
        

        contact = Contact(
            client = client,
            phoneNumber = "9990000000",
            attributes = {
                "name" : "John",
                "age": 99,
            },
        )
        contact1 = Contact(
            client = client,
            phoneNumber = "9990000001",
            attributes = {
                "name" : "John",
                "age": 99,
            },
        )

        campaign.add_contact(contact)
        campaign.add_contact(contact1)

        campaign.run()



import os
import sys
import unittest

from dotenv import load_dotenv

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from tingting.client import TingTingClient, Voice

load_dotenv()

BASE_URL = os.environ.get("BASE_URL")
TOKEN = os.environ.get("TOKEN")

class TestTingTingClient(unittest.TestCase):
    def test_get_voices(self):
        client = TingTingClient(BASE_URL,TOKEN)
        voices = client.get_voices()

        assert type(voices) is list
        assert voices.__len__() > 0
        
        for voice in voices:
            assert type(voice) is Voice

    def test_get_voice_by_id(self):
        client = TingTingClient(BASE_URL,TOKEN)
        id = 1
        voice = client.get_voice_by_id(id)

        assert type(voice) is Voice
        assert voice.id == id

    def test_get_voice_by_name(self):
        client = TingTingClient(BASE_URL,TOKEN)
        name = "rija"
        voice = client.get_voice_by_name(name)

        assert type(voice) is Voice
        assert voice.displayName == name




import os
import sys
import unittest

from dotenv import load_dotenv

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from tingting import (
    Campaign,
    CampaignStatus,
    Category,
    Contact,
    Services,
    TingTingClient,
)

load_dotenv()

BASE_URL = os.environ.get("BASE_URL")
TOKEN = os.environ.get("TOKEN")

class TestTingTingClient(unittest.TestCase):
    def test_campaign_test_voice(self):
        client = TingTingClient(BASE_URL,TOKEN)
        dialers = client.get_dialers()
        voices = client.get_voices()
        assert voices.__len__() > 0
        voice = voices[0]

        campaign = Campaign(
            client = client,
            name = "Test Campaign Test Voice",
            services = Services.PHONE,
            status = CampaignStatus.NOT_STARTED,
            message = "Phone Campaign Message",
            category = Category.TEXT,
            dialers = dialers,
            voice = voice,
            draft = False,
            lengthFactor = 1,
        )
        campaign.create()

        assert type(campaign) is Campaign
        assert type(campaign.id) is int

        contact = Contact(
            client = client,
            phoneNumber = "9990000000",
            attributes = {
                "name" : "John",
                "age": 99,
            },
        )

        campaign.add_contact(contact)

        audio_url = campaign.test_voice()
        assert audio_url.startswith("\"https://")


from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from .client import TingTingClient
    
class Dialer:
    @classmethod
    def from_data(cls, client:"TingTingClient",data:dict) -> "Dialer":
        return Dialer(
            client = client,
            id = data.get("id"),
            phoneNumber = data.get("phone_number"),
        )

    def __init__(self,
        client:"TingTingClient",
        id: Optional[int] = None,
        phoneNumber: str = "",
    ):
        self._client = client
        self.id = id
        self.phoneNumber = phoneNumber
    
    def __str__(self) -> str:
        return f"{self.phoneNumber}"
    
    def json(self) -> dict:
        return {
            "id": self.id,
            "phone_number": self.phoneNumber,
        }



import os
import sys
import unittest

from dotenv import load_dotenv

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from tingting.client import Campaign, TingTingClient

load_dotenv()

BASE_URL = os.environ.get("BASE_URL")
TOKEN = os.environ.get("TOKEN")

class TestTingTingClient(unittest.TestCase):
    def test_get_campaign_list(self):
        client = TingTingClient(BASE_URL,TOKEN)
        campaigns = client.get_campaigns()

        assert type(campaigns) is list
        assert campaigns.__len__() > 0

        for campaign in campaigns:
            assert type(campaign) is Campaign
    
    def test_get_campaign_list_by_name(self):
        client = TingTingClient(BASE_URL,TOKEN)
        campaigns = client.get_campaigns_by_name("t")

        assert type(campaigns) is list
        if campaigns.__len__() > 0:
            assert type(campaigns[0]) is Campaign
    
    def test_get_campaign_list_with_filter(self):
        client = TingTingClient(BASE_URL,TOKEN)
        campaigns = client.get_campaigns_with_filter()

        assert type(campaigns) is list
        if campaigns.__len__() > 0:
            assert type(campaigns[0]) is Campaign
    
    def test_get_campaign(self):
        client = TingTingClient(BASE_URL,TOKEN)
        campaign = client.get_campaign(723)

        assert type(campaign) is Campaign



from .campaign import Campaign  # noqa: F401
from .client import TingTingClient  # noqa: F401
from .contact import Contact  # noqa: F401
from .dialer import Dialer  # noqa: F401
from .endpoint import Endpoint  # noqa: F401
from .enums import *  # noqa: F401,F403
from .survey_step import SurveyStep  # noqa: F401
from .user import User  # noqa: F401
from .voice import Voice  # noqa: F401



from typing import TYPE_CHECKING, Optional

from .enums import ContactStatus

if TYPE_CHECKING:
    from .client import TingTingClient


class Contact:
    @classmethod
    def from_data(cls, client:"TingTingClient",campaignId:int,data:dict) -> "Contact":
        return Contact(
            client = client,
            id = data.get("id"),
            phoneNumber = data.get("number"),
            campaignId = campaignId,
            status = ContactStatus.from_str(data.get("status")),
            duration = data.get("call_duration"),
            playback = data.get("playback"),
            creditConsumed = data.get("credit_consumed"),
            creditConsumedSMS = data.get("credit_consumed_SMS"),
            carrier = data.get("carrier"),
            attributes = data.get("attributes"),
        )

    def __init__(self,
        client:"TingTingClient",
        id:Optional[int] = None,
        phoneNumber: str = "",
        campaignId: int = 0,
        status: ContactStatus = ContactStatus.NOT_STARTED,
        duration: str = "",
        playback: str = "",
        creditConsumed: int = 0,
        creditConsumedSMS: int = 0,
        carrier: str = "other",
        attributes: dict = None,
    ):
        self._client = client
        self.id = id
        self.phoneNumber = phoneNumber
        self.campaignId = campaignId
        self.status = status
        self.duration = duration
        self.playback = playback
        self.creditConsumed = creditConsumed
        self.creditConsumedSMS = creditConsumedSMS
        self.carrier = carrier
        self.attributes = attributes
    
    def __str__(self):
        return f"{self.phoneNumber}"
    
    def json(self):
        return {
            "id": self.id,
            "number": self.phoneNumber,
            "campaign_id": self.campaignId,
            "status": self.status.value,
            "duration": self.duration,
            "playback": self.playback,
            "credit_consumed": self.creditConsumed,
            "credit_consumed_SMS": self.creditConsumedSMS,
            "carrier": self.carrier,
            "other_variables": self.attributes,
        }
    
    def update_from(self, other: "Contact"):
        self.id = other.id
        self.phoneNumber = other.phoneNumber
        self.campaignId = other.campaignId
        self.status = other.status
        self.duration = other.duration
        self.playback = other.playback
        self.creditConsumed = other.creditConsumed
        self.creditConsumedSMS = other.creditConsumedSMS
        self.carrier = other.carrier
        self.attributes = other.attributes

    def delete(self):
        return self._client.delete_contact(self)

    def save(self):
        if self.id:
            return self.update()
        return self.create()

    def create(self) -> "Contact":
        json = self.json()
        created = self._client.add_contact(self.campaignId,json)
        self.update_from(created)
        return self

    def update(self):
        return self._client.update_contact(self)

    def test_call(self):
        return self._client.test_call(self)

    def test_survey(self):
        return self._client.test_survey(self)


from typing import IO, List, Optional

import requests

from .campaign import Campaign
from .contact import Contact
from .dialer import Dialer
from .endpoint import Endpoint
from .enums import CampaignStatus
from .survey_step import SurveyStep
from .user import User
from .voice import Voice


class TingTingClient:
    def __init__(self, baseUrl: str, token: str = None):
        self.baseUrl = baseUrl
        self.token = token

    def _headers(self):
        return {
            "Authorization": f"Bearer {self.token}" if self.token else "",
        }

    def get_data(self, endpoint: str, params: dict = None):
        response = requests.get(f"{self.baseUrl}/{endpoint}", headers=self._headers(), params=params)
        response.raise_for_status()
        return response.json()

    def post_data(self, endpoint: str, data: dict):
        response = requests.post(f"{self.baseUrl}/{endpoint}", headers=self._headers(), json=data)
        response.raise_for_status()
        return response.json()


    def get_user(self) -> User:
        response = requests.get(f"{self.baseUrl}/{Endpoint.USER_INFO.value}", headers=self._headers())
        response.raise_for_status()
        data = response.json()
        return User.from_data(self,data)


    def get_dialers(self) -> List[Dialer]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.DIALER_LIST.value}", headers=self._headers())
        response.raise_for_status()
        data:list = response.json()
        dialers = [Dialer.from_data(self,dialer) for dialer in data]
        return dialers

    def get_dialer_by_id(self,id) -> Optional[Dialer]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.DIALER_LIST.value}", headers=self._headers())
        response.raise_for_status()
        data:list = response.json()

        filteredData = list(filter(lambda dialer: dialer.get("id")==id,data))

        if filteredData:
            return Dialer.from_data(self,filteredData[0])
        
        return None

    def get_dialer_by_number(self,phoneNumber) -> Optional[Dialer]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.DIALER_LIST.value}", headers=self._headers())
        response.raise_for_status()
        data:list = response.json()

        filteredData = list(filter(lambda dialer: dialer.get("phone_number")==phoneNumber,data))

        if filteredData:
            return Dialer.from_data(self,filteredData[0])
        
        return None


    def get_voices(self) -> List[Voice]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.VOICE_LIST.value}", headers=self._headers())
        response.raise_for_status()
        data:list = response.json()
        voices = [Voice.from_data(self,voice) for voice in data]
        return voices

    def get_voice_by_id(self,id:int) -> Optional[Voice]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.VOICE_LIST.value}", headers=self._headers())
        response.raise_for_status()
        data:list = response.json()

        filteredData = list(filter(lambda voice: voice.get("id")==id,data))

        if filteredData:
            return Voice.from_data(self,filteredData[0])
        
        return None

    def get_voice_by_name(self,name:str) -> Optional[Voice]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.VOICE_LIST.value}", headers=self._headers())
        response.raise_for_status()
        data:list = response.json()

        filteredData = list(filter(lambda voice: voice.get("voice_display_name","").lower()==name.lower(),data))

        if filteredData:
            return Voice.from_data(self,filteredData[0])
        
        return None


    def get_campaigns(self) -> List[Campaign]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.CAMPAIGN_LIST.value}", headers=self._headers())
        response.raise_for_status()
        data = response.json()
        campaigns = []
        if "results" in data:
            campaigns = [Campaign.from_data(self,camp) for camp in data["results"]]
        
        return campaigns

    def get_campaigns_by_name(self,name:str) -> List[Campaign]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.CAMPAIGN_LIST.value}", headers=self._headers(), params={
            "search":name
        })
        response.raise_for_status()
        data = response.json()
        campaigns = []
        if "results" in data:
            campaigns = [Campaign.from_data(self,camp) for camp in data["results"]]
        
        return campaigns

    def get_campaigns_with_filter(self,name:str=None,status:CampaignStatus=None,draft:bool=False,sort_by=None,from_date=None,to_date=None) -> List[Campaign]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.CAMPAIGN_LIST.value}", headers=self._headers(), params={
            "search":name,
            "status":status,
            "draft":draft,
            "sort_by":sort_by,
            "from_date":from_date,
            "to_date":to_date,
        })
        response.raise_for_status()
        data = response.json()
        campaigns = []
        if "results" in data:
            campaigns = [Campaign.from_data(self,camp) for camp in data["results"]]
        
        return campaigns
    
    def get_campaign(self,id:int) -> List[Campaign]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.CAMPAIGN_GET(id)}", headers=self._headers())
        response.raise_for_status()
        data = response.json()
        return Campaign.from_data(self,data)


    def create_campaign(self,data:dict)->Campaign:
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_CREATE.value}",
            headers=self._headers(),
            json=data,
        )
        response.raise_for_status()
        data = response.json()
        return Campaign.from_data(self,data)

    def update_campaign(self,campaign:Campaign):
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_UPDATE(campaign.id)}",
            headers=self._headers(),
            data=campaign.json(),
        )
        response.raise_for_status()
        # data = response.json()
    
    def delete_campaign(self,campaign:Campaign):
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_DELETE(campaign.id)}",
            headers=self._headers(),
            data=campaign.json(),
        )
        response.raise_for_status()
        # data = response.json()
    
    def test_voice_campaign(self,campaign:Campaign) -> str:
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_TEST_VOICE(campaign.id)}",
            headers=self._headers(),
            data={
                "voice_input": campaign.voiceId,
                "message": campaign.message,
            },
        )
        response.raise_for_status()
        # data = response.json()
        return response.text
    
    def run_campaign(self,campaign:Campaign):
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_RUN_CAMPAIGN(campaign.id)}",
            headers=self._headers(),
        )
        response.raise_for_status()
        # data = response.json()
    
    def stop_campaign(self,campaign:Campaign):
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_STOP_CAMPAIGN(campaign.id)}",
            headers=self._headers(),
            data=campaign.json(),
        )
        response.raise_for_status()
        # data = response.json()
    
    def get_campaign_report(self,campaign:Campaign):
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_REPORT(campaign.id)}",
            headers=self._headers(),
            data=campaign.json(),
        )
        response.raise_for_status()
        # data = response.json()


    def add_contact(self,campaignId:int,data:dict) -> Contact:
        response = requests.post(f"{self.baseUrl}/{Endpoint.CONTACT_ADD(campaignId)}",
            headers=self._headers(),
            json=data,
        )
        response.raise_for_status()
        data = response.json()
        return Contact.from_data(self,campaignId,data)

    def add_contact_file(self,campaign:Campaign,file:IO[bytes]):
        response = requests.patch(f"{self.baseUrl}/{Endpoint.CONTACT_ADD_BULK(campaign.id)}",
            headers=self._headers(),
            files = {"bulk_file": ("contacts.csv", file, "text/csv")}
        )
        response.raise_for_status()
        # data = response.json()
    
    def get_contacts(self,campaign:Campaign)->List[Contact]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.CONTACT_LIST(campaign.id)}",
            headers=self._headers(),
        )
        response.raise_for_status()
        # data = response.json()
    
    def update_contact(self,contact:Contact):
        response = requests.patch(f"{self.baseUrl}/{Endpoint.CONTACT_UPDATE(contact.id)}",
            headers=self._headers(),
            data=contact.json()
        )
        response.raise_for_status()
        # data = response.json()
    
    def delete_contact(self,contact:Contact):
        response = requests.delete(f"{self.baseUrl}/{Endpoint.CONTACT_DELETE(contact.id)}",
            headers=self._headers(),
        )
        response.raise_for_status()
        # data = response.json()

    def get_contact_attribute(self,contact:Contact):
        response = requests.get(f"{self.baseUrl}/{Endpoint.CONTACT_DELETE(contact.id)}",
            headers=self._headers(),
        )
        response.raise_for_status()
        # data = response.json()

    def update_contact_attribute(self,contact:Contact):
        response = requests.delete(f"{self.baseUrl}/{Endpoint.CONTACT_DELETE(contact.id)}",
            headers=self._headers(),
            data=contact.json(),
        )
        response.raise_for_status()
        # data = response.json()

    def test_call(self,contact:Contact):
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_DEMO_CALL(contact.id)}",
            headers=self._headers(),
        )
        response.raise_for_status()
        # data = response.json()
    
    def get_survey_steps(self, campaign:Campaign) -> List[SurveyStep]:
        response = requests.get(f"{self.baseUrl}/{Endpoint.CAMPAIGN_GET_SURVEY_STEPS(campaign.id)}", headers=self._headers())
        response.raise_for_status()
        data = response.json()
        surveySteps = []
        if "results" in data:
            surveySteps = [SurveyStep.from_data(self,step) for step in data["results"]]
        
        return surveySteps

    def create_survey_campaign(self, data:dict,files:list) -> Campaign:
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_CREATE.value}",
            headers=self._headers(),
            json=data,
        )
        response.raise_for_status()
        data_ = response.json()
        campaign = Campaign.from_data(self,data_)

        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_CREATE_SURVEY_STEPS(campaign.id)}",
            headers=self._headers(),
            data=data,
            files=files,
        )

        response.raise_for_status()
        data = response.json()

        surveySteps = self.get_survey_steps(campaign)
        for surveyStep in surveySteps:
            campaign.add_survey_step(surveyStep)
        
        return campaign

    def update_survey_campaign(self, campaign:Campaign,files:list) -> Campaign:
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_UPDATE(campaign.id)}",
            headers=self._headers(),
            data=campaign.json(),
        )
        response.raise_for_status()
        data_ = response.json()
        campaign = Campaign.from_data(self,data_)

        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_UPDATE_SURVEY_STEPS(campaign.id)}",
            headers=self._headers(),
            data=campaign.json(),
            files=files,
        )

        response.raise_for_status()
        _ = response.json()

        surveySteps = self.get_survey_steps(campaign)
        for surveyStep in surveySteps:
            campaign.add_survey_step(surveyStep)
        
        return campaign
    
    def run_survey(self,campaign:Campaign):
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_RUN_SURVEY(campaign.id)}",
            headers=self._headers(),
        )
        response.raise_for_status()
        # data = response.json()
    
    def test_survey(self,contact:Contact):
        response = requests.post(f"{self.baseUrl}/{Endpoint.CAMPAIGN_TEST_SURVEY(contact.id)}",
            headers=self._headers(),
        )
        response.raise_for_status()
        # data = response.json()
    



import os
import sys
import unittest

from dotenv import load_dotenv

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from tingting.client import Dialer, TingTingClient

load_dotenv()

BASE_URL = os.environ.get("BASE_URL")
TOKEN = os.environ.get("TOKEN")

class TestTingTingClient(unittest.TestCase):
    def test_get_dialers(self):
        client = TingTingClient(BASE_URL,TOKEN)
        dialers = client.get_dialers()

        assert type(dialers) is list
        assert dialers.__len__() > 0

        for dialer in dialers:
            assert type(dialer) is Dialer

    def test_get_dialer_by_id(self):
        client = TingTingClient(BASE_URL,TOKEN)
        id = 10
        dialer = client.get_dialer_by_id(id)

        assert type(dialer) is Dialer
        assert dialer.id == id

    def test_get_dialer_by_name(self):
        client = TingTingClient(BASE_URL,TOKEN)
        phoneNumber = "+97715970969"
        dialer = client.get_dialer_by_number(phoneNumber)

        assert type(dialer) is Dialer
        assert dialer.phoneNumber == phoneNumber



from enum import Enum


class Endpoint(Enum):
    USER_INFO = "api/v1/auths/user-profile/"
    DIALER_LIST = "api/v1/active-broker-phone/"
    VOICE_LIST = "api/v1/voice-models/"

    CAMPAIGN_LIST = "api/v1/campaign/"
    CAMPAIGN_CREATE = "api/v1/campaign/create/"
    CAMPAIGN_GET = lambda campaignId : f"api/v1/campaign/{campaignId}/"
    CAMPAIGN_UPDATE = lambda campaignId : f"api/v1/campaign/{campaignId}/"
    CAMPAIGN_DELETE = lambda campaignId : f"api/v1/campaign/{campaignId}/"
    CAMPAIGN_TEST_VOICE = lambda campaignId : f"api/v1/test-speak/riri/{campaignId}/"
    CAMPAIGN_RUN_CAMPAIGN = lambda campaignId : f"api/v1/run-campaign/{campaignId}/"
    CAMPAIGN_STOP_CAMPAIGN = lambda campaignId : f"api/v1/campaign/emergency-stop/{campaignId}/"
    CAMPAIGN_REPORT = lambda campaignId : f"api/v1/download/report/{campaignId}/"

    CAMPAIGN_GET_SURVEY_STEPS = lambda campaignId : f"api/v1/campaign/create/{campaignId}/survey/"
    CAMPAIGN_CREATE_SURVEY_STEPS = lambda campaignId : f"api/v1/campaign/create/{campaignId}/survey/"
    CAMPAIGN_UPDATE_SURVEY_STEPS = lambda campaignId : f"api/v1/campaign/create/{campaignId}/survey/"
    CAMPAIGN_DELETE_SURVEY_STEPS = lambda campaignId : f"api/v1/campaign/create/{campaignId}/survey/"

    CAMPAIGN_RUN_SURVEY = lambda campaignId : f"api/v1/run-survey/{campaignId}/"
    CAMPAIGN_TEST_SURVEY = lambda contactId : f"api/v1/test-survey/{contactId}/"

    CONTACT_ADD = lambda campaignId : f"api/v1/campaign/{campaignId}/add-contact/"
    CONTACT_ADD_BULK = lambda campaignId : f"api/v1/campaign/create/{campaignId}/detail/"
    CONTACT_LIST = lambda campaignId: f"api/v1/campaign-detail/{campaignId}/"
    CONTACT_UPDATE = lambda contactId: f"api/v1/phone-number/update/{contactId}/"
    CONTACT_DELETE = lambda contactId: f"api/v1/phone-number/delete/{contactId}/"
    CAMPAIGN_DEMO_CALL = lambda contactId : f"api/v1/demo-call/{contactId}/"

    CONTACT_ATTRIBUTES = lambda id: f"api/v1/campaign/{id}/attributes/"
    CONTACT_ATTRIBUTES_UPDATE = lambda id: f"api/v1/campaign/{id}/attributes/"




from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from .client import TingTingClient
    
class Voice:
    @classmethod
    def from_data(cls, client:"TingTingClient",data:dict) -> "Voice":
        return Voice(
            client = client,
            id = data.get("id"),
            displayName = data.get("voice_display_name"),
            internalName = data.get("voice_internal_name"),
            isPremium = data.get("is_premium"),
        )

    def __init__(self,
        client:"TingTingClient",
        id: Optional[int] = None,
        displayName: str = "",
        internalName: str = "",
        isPremium: bool = False,
    ):
        self._client = client
        self.id = id
        self.displayName = displayName
        self.internalName = internalName
        self.isPremium = isPremium
    
    def __str__(self) -> str:
        return f"{self.displayName}"
    
    def json(self) -> dict:
        return {
            "id": self.id,
            "voice_display_name": self.displayName,
            "voice_internal_name": self.internalName,
            "is_premium": self.isPremium,
        }